% For use with Matlab
% Copyright (C) 2019 -- Jérôme Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araújo <rui@isr.uc.pt>
% Jérôme Mendes, Francisco Souza, and Rui Araújo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jérôme Mendes, Francisco Souza and Rui Araújo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [MF_center, nearestMF] = LocateMF_Gravity(error_controller_estimation_all, data_window, j, x)
%% Description
% This function obtains the position/center of the new (candidate) membership function for variable j (Equation (22))

%% Function inputs
% error_controller_estimation_all - estimated control error for each rule
% data_window - temporal sliding window
% j - input variable which has the maximal estimated control error (i.e. where the new MF will be added)
% x - Struct with all fuzzy control parameters

%% Function output
% MF_center - position/center of the new (candidate) membership function
% nearestMF -  the nearest membership function from candidate membership function

error_size = size(error_controller_estimation_all, 2);% Memory allocation with zeros

%%  Distribution of control estimation error - equation (22)

num = 0.0;  % Auxiliary variable
den = 0.0;  % Auxiliary variable

for m=1:error_size % for all temporal sliding window
    
    num = num + m*error_controller_estimation_all(m)^2; % numerator of equation (22)
    den = den + error_controller_estimation_all(m)^2; % denominator of equation (22)
    
end

gravity_center_error = num/den; % equation (22)

if j == 1
    var = size(data_window, 1)-1; % if the selected variable is ref (j=1), this variable is stored at end-1 at data_window
else
    var = j;
end

MF_center =  data_window(var, ceil(gravity_center_error) ); % position/center of the new (candidate) membership function

%% Obtain the distance between the center of the candidate membership function and the other ones
n_rules = size( x(j).Rules,1 );% number of rules on input variable j
dist = zeros(1,n_rules);% Memory allocation with zeros

for i=1:n_rules
    dist(i) = abs(x(j).Rules(i,2) - MF_center);
end

nearestMF = find( dist == min(dist), 1 ); % the nearest membership function from candidate membership function

end